--[[
Mod: ResourceMenu
Author: hllf & JLove
Version: 27

Intended as an accessibility mod. Places game resources and other related information in a menu.
Use the mod importer to import this mod.
--]]

OnControlPressed{ "MoveUp", function(triggerArgs)
    if IsScreenOpen("TraitTrayScreen") then
        CloseAdvancedTooltipScreen()
        OpenAssesResourceShowerMenu()
    end
end}

OnControlPressed{ "Attack3", function(triggerArgs)
    local curMap = GetMapName({})
    if curMap:find("DeathArea", 1, true) == 1 or curMap:find("E_", 1, true) == 1 then
        OpenAssesResourceShowerMenu()
    end
end}

OnControlPressed{ "AdvancedTooltip", function(triggerArgs)
    local curMap = GetMapName({})
    if curMap:find("DeathArea", 1, true) == 1 or curMap:find("E_", 1, true) == 1 then
        return
    end
    if IsScreenOpen("Codex") then
        return
    end
    if ( not IsInputAllowed({}) and not GameState.WaitingForChoice ) or ( CurrentDeathAreaRoom ~= nil and CurrentDeathAreaRoom.ShowResourceUIOnly ) then
        return
    end
    if ( CurrentRun ~= nil and CurrentRun.Hero ~= nil and ( IsEmpty( CurrentRun.Hero.Traits ) and GetTotalSpentMetaPoints() == 0 ) ) then
        if ScreenAnchors.TraitTrayScreen ~= nil and ScreenAnchors.TraitTrayScreen.CanClose then
            CloseAdvancedTooltipScreen()
        else
            ShowDepthCounter()
            ShowAdvancedTooltip( { AutoPin = false, } )
        end
    end
end}

function OpenAssesResourceShowerMenu(usee)
	local screen = { Components = {} }
	screen.Name = "BlindAccesibilityResourceMenu"

	if IsScreenOpen( screen.Name ) then
		return
	end
    OnScreenOpened({ Flag = screen.Name, PersistCombatUI = false })
    HideCombatUI()
	FreezePlayerUnit()
	SetConfigOption({ Name = "FreeFormSelectWrapY", Value = false })
	SetConfigOption({ Name = "FreeFormSelectStepDistance", Value = 8 })
	SetConfigOption({ Name = "FreeFormSelectSuccessDistanceStep", Value = 8 })
	SetConfigOption({ Name = "FreeFormSelectRepeatDelay", Value = 0.6 })
	SetConfigOption({ Name = "FreeFormSelectRepeatInterval", Value = 0.1 })
	SetConfigOption({ Name = "FreeFormSelecSearchFromId", Value = 0 })

	PlaySound({ Name = "/SFX/Menu Sounds/ContractorMenuOpen" })
	local components = screen.Components

	components.ShopBackgroundDim = CreateScreenComponent({ Name = "rectangle01", Group = "Asses_UI" })
	
	components.CloseButton = CreateScreenComponent({ Name = "ButtonClose", Group = "Asses_UI_Backing", Scale = 0.7 })
	Attach({ Id = components.CloseButton.Id, DestinationId = components.ShopBackgroundDim.Id, OffsetX = 0, OffsetY = 440 })
	components.CloseButton.OnPressedFunctionName = "CloseAssesResourceShowerScreen"
	components.CloseButton.ControlHotkey = "Cancel"

	SetScale({ Id = components.ShopBackgroundDim.Id, Fraction = 4 })
	SetColor({ Id = components.ShopBackgroundDim.Id, Color = {0, 0, 0, 1} })
		
    CreateAssesResourceText(screen);

	screen.KeepOpen = true
	thread( HandleWASDInput, screen )
	HandleScreenInput( screen )

end

local keepsakeTraitToName = {
    ["MaxHealthKeepsakeTrait"] = "Cerberus's Old Spiked Collar",
    ["DirectionalArmorTrait"] = "Achilles's Myrmidon Bracer",
    ["BackstabAlphaStrikeTrait"] = "Nyx's Black Shawl",
    ["PerfectClearDamageBonusTrait"] = "Thanatos's Pierced Butterfly",
    ["ShopDurationTrait"] = "Charon's Bone Hourglass",
    ["BonusMoneyTrait"] = "Hypnos's Chthonic Coin Purse",
    ["LowHealthDamageTrait"] = "Meg's Skull Earring",
    ["DistanceDamageTrait"] = "Orpheus's Distant Memory",
    ["LifeOnUrnTrait"] = "Dusa's Harpy Feather Duster",
    ["ReincarnationTrait"] = "Skelly's Lucky Tooth",
    ["ForceZeusBoonTrait"] = "Zeus's Thunder Signet",
    ["ForcePoseidonBoonTrait"] = "Poseidon's Conch Shell",
    ["ForceAthenaBoonTrait"] = "Athena's Owl Pendant",
    ["ForceAphroditeBoonTrait"] = "Aphrodite's Eternal Rose",
    ["ForceAresBoonTrait"] = "Ares's Blood-Filled Vial",
    ["ForceArtemisBoonTrait"] = "Artemis's Adamant Arrowhead",
    ["ForceDionysusBoonTrait"] = "Dionysus's Overflowing Cup",
    ["FastClearDodgeBonusTrait"] = "Hermes's Lambent Plume",
    ["ForceDemeterBoonTrait"] = "Demeter's Frostbitten Horn",
    ["ChaosBoonTrait"] = "Chaos's Cosmic Egg",
    ["VanillaTrait"] = "Sisyphus's Shattered Shackle",
    ["ShieldBossTrait"] = "Eurydice's Evergreen Acorn",
    ["ShieldAfterHitTrait"] = "Patroclus's Broken Spearpoint",
    ["ChamberStackTrait"] = "Persephone's Pom Blossom",
    ["HadesShoutKeepsake"] = "Hades's Sigil of the Dead",
    ["FuryAssistTrait"] = "Battie",
    ["ThanatosAssistTrait"] = "Mort",
    ["SkellyAssistTrait"] = "Rib",
    ["SisyphusAssistTrait"] = "Shady",
    ["DusaAssistTrait"] = "Fidi",
    ["AchillesPatroclusAssistTrait"] = "Antos",
}

function CreateAssesResourceText(screen)
    local xPos = -100
    local startY = -105
    local yIncrement = 30
    local curY = startY
    local components = screen.Components

    local codeToDisplayName = {
        {["key"] = "MetaPoints", ["value"] = "Darkness"},
        {["key"] = "Gems", ["value"] = "Gemstones"},
        {["key"] = "LockKeys", ["value"] = "Chthonic Keys"},
        {["key"] = "GiftPoints", ["value"] = "Nectar"},
        {["key"] = "SuperGems", ["value"] = "Diamonds"},
        {["key"] = "SuperGiftPoints", ["value"] = "Ambrosia"},
        {["key"] = "SuperLockKeys", ["value"] = "Titan Blood"},
    }
    local nameToWeaponDisplayInfo = {
        SwordWeapon = {
            Name = "Stygian Blade",
            Aspect = {"Zagreus", "Nemesis", "Poseidon", "Arthur"}
        },
        BowWeapon = {
            Name = "Heart-Seeking Bow",
            Aspect = {"Zagreus", "Chiron", "Hera", "Rama"}
        },
        SpearWeapon = {
            Name = "Eternal Spear",
            Aspect = {"Zagreus", "Achilles", "Hades", "Guan Yu"}
        },
        GunWeapon = {
            Name = "Adamant Rail",
            Aspect = {"Zagreus", "Eris", "Hestia", "Lucifer"}
        },
        FistWeapon = {
            Name = "Twin Fists",
            Aspect = {"Zagreus", "Talos", "Demeter", "Gilgamesh"}
        },
        ShieldWeapon = {
            Name = "Shield of Chaos",
            Aspect = {"Zagreus", "Chaos", "Zeus", "Beowulf"}
        },
    }

    if not string.find(GetMapName({}), "RoomPreRun") and GetMapName({}):find("DeathArea", 1, true) ~= 1 and GetMapName({}):find("E_", 1, true) ~= 1 then
        CreateTextBox({
            Id =components.ShopBackgroundDim.Id,
            Text = "Chamber: " .. GetRunDepth( CurrentRun ),
            FontSize = 24,
            Width = 360,
            OffsetX = xPos,
            OffsetY = curY,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group="Asses_UI",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        curY = curY + yIncrement
    end
    CreateTextBox({
        Id =components.ShopBackgroundDim.Id,
        Text = "Health: " .. (CurrentRun.Hero.Health or 0) .. "/" .. (CurrentRun.Hero.MaxHealth or 0),
        FontSize = 24,
        Width = 360,
        OffsetX = xPos,
        OffsetY = curY,
        Color = Color.White,
        Font = "AlegreyaSansSCLight",
        Group="Asses_UI",
        ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
        Justification = "Left",
    })
    curY = curY + yIncrement
    if not string.find(GetMapName({}), "RoomPreRun") and GetMapName({}):find("DeathArea", 1, true) ~= 1 then
        CreateTextBox({
            Id =components.ShopBackgroundDim.Id,
            Text = "Death Defiances: " .. (#CurrentRun.Hero.LastStands or 0),
            FontSize = 24,
            Width = 360,
            OffsetX = xPos,
            OffsetY = curY,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group="Asses_UI",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        curY = curY + yIncrement
        if GetNumMetaUpgrades( "RerollMetaUpgrade" ) + GetNumMetaUpgrades("RerollPanelMetaUpgrade") > 0 then
            CreateTextBox({
                Id =components.ShopBackgroundDim.Id,
                Text = "Fated Rerolls: " .. (CurrentRun.NumRerolls or 0),
                FontSize = 24,
                Width = 360,
                OffsetX = xPos,
                OffsetY = curY,
                Color = Color.White,
                Font = "AlegreyaSansSCLight",
                Group="Asses_UI",
                ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
                Justification = "Left",
            })
            curY = curY + yIncrement
        end
        CreateTextBox({
            Id =components.ShopBackgroundDim.Id,
            Text = "Obols: " .. ((CurrentRun or {Money = 0}).Money or 0),
            FontSize = 24,
            Width = 360,
            OffsetX = xPos,
            OffsetY = curY,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group="Asses_UI",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        curY = curY + yIncrement
    end
    if GameState.LastAwardTrait ~= nil and HeroHasTrait(GameState.LastAwardTrait) then
        CreateTextBox({
            Id =components.ShopBackgroundDim.Id,
            Text = "Keepsake: " .. (keepsakeTraitToName[GameState.LastAwardTrait] or GameState.LastAwardTrait),
            FontSize = 24,
            Width = 720,
            OffsetX = xPos,
            OffsetY = curY,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group="Asses_UI",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        curY = curY + yIncrement
        if IsKeepsakeMaxed(GameState.LastAwardTrait) then
            levelText = GetKeepsakeLevel(GameState.LastAwardTrait) .. " (maxed)"
        else
            levelText = GetKeepsakeLevel(GameState.LastAwardTrait) .. " (" .. GetKeepsakeChambersToNextLevel(GameState.LastAwardTrait) .. "encounters to next level)"
        end
        CreateTextBox({
            Id =components.ShopBackgroundDim.Id,
            Text = "Keepsake Level: " .. levelText,
            FontSize = 24,
            Width = 720,
            OffsetX = xPos,
            OffsetY = curY,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group="Asses_UI",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        curY = curY + yIncrement
    end
    if GameState.LastAssistTrait ~= nil and HeroHasTrait(GameState.LastAssistTrait) then
        local remainingUses = 0
        for i, traitData in pairs( CurrentRun.Hero.Traits ) do
            if traitData.AddAssist and traitData.RemainingUses ~= nil then
                remainingUses = traitData.RemainingUses
            end
        end
        CreateTextBox({
            Id =components.ShopBackgroundDim.Id,
            Text = "Companion: " .. (keepsakeTraitToName[GameState.LastAssistTrait] or GameState.LastAssistTrait) .. " (Level " .. GetKeepsakeLevel(GameState.LastAssistTrait) .. ", Uses " .. remainingUses .. ")",
            FontSize = 24,
            Width = 720,
            OffsetX = xPos,
            OffsetY = curY,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group="Asses_UI",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        curY = curY + yIncrement
    end
    if GetEquippedWeapon() ~= nil then
        local weapon = GetEquippedWeapon()
        local weaponAspect = GetEquippedWeaponTraitIndex(weapon)
        local weaponAspectLevel = GetWeaponUpgradeLevel(weapon, weaponAspect)
        weaponAspect = nameToWeaponDisplayInfo[weapon].Aspect[weaponAspect]
        weapon = nameToWeaponDisplayInfo[weapon].Name
        CreateTextBox({
            Id =components.ShopBackgroundDim.Id,
            Text = "Weapon: " .. weapon .. ", Aspect: " .. weaponAspect .. ", Level: " .. weaponAspectLevel,
            FontSize = 24,
            Width = 720,
            OffsetX = xPos,
            OffsetY = curY,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group="Asses_UI",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        curY = curY + yIncrement
    end
    for kKey,vValue in pairs(codeToDisplayName) do
		k = vValue["key"]
		v = vValue["value"]
		CreateTextBox({
            Id =components.ShopBackgroundDim.Id,
            Text = v .. ": " .. (GameState.Resources[k] or 0),
            FontSize = 24,
            Width = 360,
            OffsetX = xPos,
            OffsetY = curY,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group="Asses_UI",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        curY = curY + yIncrement
    end
end

function CloseAssesResourceShowerScreen( screen, button )
	SetConfigOption({ Name = "FreeFormSelectStepDistance", Value = 16 })
	SetConfigOption({ Name = "FreeFormSelectSuccessDistanceStep", Value = 8 })
	SetConfigOption({ Name = "FreeFormSelectRepeatDelay", Value = 0.0 })

	PlaySound({ Name = "/SFX/Menu Sounds/ContractorMenuClose" })
	CloseScreen( GetAllIds( screen.Components ) )

	UnfreezePlayerUnit()
	screen.KeepOpen = false
	OnScreenClosed({ Flag = screen.Name })
end