--[[
Mod: RelationshipMenu
Author: hllf
Version: 26

Intended as an accessibility mod. Places Zagreus's relationship status with other NPCs and other related information in a menu.
Use the mod importer to import this mod.
--]]

OnControlPressed{ "MoveUp", function(triggerArgs)
    if IsScreenOpen("Codex") and GameState.Gift ~= nil and NumNPCWithHearts() > 0 then
        wait(0.1)
        CloseCodexScreen()
        OpenRelationshipMenu()
    end
end}

function OpenRelationshipMenu(usee)
    local screen = { Components = {} }
    screen.Name = "BlindAccessibilityRelationshipMenu"
    if IsScreenOpen( screen.Name ) then
        return
    end
    OnScreenOpened({ Flag = screen.Name, PersistCombatUI = false })
    HideCombatUI()
    FreezePlayerUnit()
    SetConfigOption({ Name = "FreeFormSelectWrapY", Value = false })
    SetConfigOption({ Name = "FreeFormSelectStepDistance", Value = 8 })
    SetConfigOption({ Name = "FreeFormSelectSuccessDistanceStep", Value = 8 })
    SetConfigOption({ Name = "FreeFormSelectRepeatDelay", Value = 0.6 })
    SetConfigOption({ Name = "FreeFormSelectRepeatInterval", Value = 0.1 })
    SetConfigOption({ Name = "FreeFormSelecSearchFromId", Value = 0 })
    PlaySound({ Name = "/SFX/Menu Sounds/ContractorMenuOpen" })
    local components = screen.Components
    components.ShopBackgroundDim = CreateScreenComponent({ Name = "rectangle01", Group = "Asses_UI" })
    components.CloseButton = CreateScreenComponent({ Name = "ButtonClose", Group = "Asses_UI_Backing", Scale = 0.7 })
    Attach({ Id = components.CloseButton.Id, DestinationId = components.ShopBackgroundDim.Id, OffsetX = 0, OffsetY = 440 })
    components.CloseButton.OnPressedFunctionName = "CloseRelationshipScreen"
    components.CloseButton.ControlHotkey = "Cancel"
    SetScale({ Id = components.ShopBackgroundDim.Id, Fraction = 4 })
    SetColor({ Id = components.ShopBackgroundDim.Id, Color = {0, 0, 0, 1} })
    CreateRelationshipText(screen)
    screen.KeepOpen = true
    thread( HandleWASDInput, screen )
    HandleScreenInput( screen )
end

local giftNPCNames = {
    {["key"] = "NPC_Cerberus_01", ["value"] = "Cerberus"},
    {["key"] = "NPC_Achilles_01", ["value"] = "Achilles"},
    {["key"] = "NPC_Nyx_01", ["value"] = "Nyx"},
    {["key"] = "NPC_Thanatos_01", ["value"] = "Thanatos"},
    {["key"] = "NPC_Charon_01", ["value"] = "Charon"},
    {["key"] = "NPC_Hypnos_01", ["value"] = "Hypnos"},
    {["key"] = "NPC_FurySister_01", ["value"] = "Meg"},
    {["key"] = "NPC_Orpheus_01", ["value"] = "Orpheus"},
    {["key"] = "NPC_Dusa_01", ["value"] = "Dusa"},
    {["key"] = "NPC_Skelly_01", ["value"] = "Skelly"},
    {["key"] = "ZeusUpgrade", ["value"] = "Zeus"},
    {["key"] = "PoseidonUpgrade", ["value"] = "Poseidon"},
    {["key"] = "AthenaUpgrade", ["value"] = "Athena"},
    {["key"] = "AphroditeUpgrade", ["value"] = "Aphrodite"},
    {["key"] = "AresUpgrade", ["value"] = "Ares"},
    {["key"] = "ArtemisUpgrade", ["value"] = "Artemis"},
    {["key"] = "DionysusUpgrade", ["value"] = "Dionysus"},
    {["key"] = "HermesUpgrade", ["value"] = "Hermes"},
    {["key"] = "DemeterUpgrade", ["value"] = "Demeter"},
    {["key"] = "TrialUpgrade", ["value"] = "Chaos"},
    {["key"] = "NPC_Sisyphus_01", ["value"] = "Sisyphus"},
    {["key"] = "NPC_Eurydice_01", ["value"] = "Eurydice"},
    {["key"] = "NPC_Patroclus_01", ["value"] = "Patroclus"},
    {["key"] = "NPC_Persephone_Home_01", ["value"] = "Persephone"},
    {["key"] = "NPC_Hades_01", ["value"] = "Hades"},
}

function NumNPCWithHearts()
    num = 0
    for k, v in pairs(giftNPCNames) do
        if GetGiftLevel(v["key"]) > 0 and GetMaxGiftLevel(v["key"]) > 0 then
            num = num + 1
        end
    end
    return num
end

function CreateRelationshipText(screen)
    local xPos = -100
    local startY = -505
    local yIncrement = 30
    local curY = startY
    local components = screen.Components
    for k, v in pairs(giftNPCNames) do
        npcName = v["key"]
        displayName = v["value"]
        numHearts = GetGiftLevel(npcName)
        maxHearts = GetMaxGiftLevel(npcName)
        if numHearts > 0 and maxHearts > 0 then
            requiredResourceAmount = GetNextGiftResourceQuantity(npcName)
            if requiredResourceAmount == 1 then
                requiredResource = "nectar"
                if GiftData[npcName][numHearts + 1] and GiftData[npcName][numHearts + 1].RequiredResource == "SuperGiftPoints" then
                    requiredResource = "ambrosia"
                end
            else
                requiredResource = requiredResourceAmount .. " nectar"
                if GiftData[npcName][numHearts + 1] and GiftData[npcName][numHearts + 1].RequiredResource == "SuperGiftPoints" then
                    requiredResource = requiredResourceAmount .. " ambrosia"
                end
            end
            completelyUnlocked = IsGiftBarCompletelyUnlocked( npcName )
            lockedLevel = GetLockedLevel(npcName)
            displayText = displayName .. ": " .. numHearts
            if completelyUnlocked then
                displayText = displayText .. "/" .. maxHearts
            end
            displayText = displayText .. " heart"
            if (completelyUnlocked and maxHearts > 1) or numHearts > 1 then
                displayText = displayText .. "s"
            end
            if not completelyUnlocked and numHearts == lockedLevel - 1 then
                displayText = displayText .. ", locked"
            end
            if completelyUnlocked then
                if numHearts == maxHearts then
                    displayText = displayText .. ", forged bond"
                else
                    displayText = displayText .. ", requires " .. requiredResource
                end
            end
            CreateTextBox({
                Id =components.ShopBackgroundDim.Id,
                Text = displayText,
--                Text = v .. ": " .. (GameState.Resources[k] or 0),
                FontSize = 24,
                Width = 720,
                OffsetX = xPos,
                OffsetY = curY,
                Color = Color.White,
                Font = "AlegreyaSansSCLight",
                Group="Asses_UI",
                ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
                Justification = "Left",
            })
            curY = curY + yIncrement
        end
    end
end

function CloseRelationshipScreen( screen, button )
    SetConfigOption({ Name = "FreeFormSelectStepDistance", Value = 16 })
    SetConfigOption({ Name = "FreeFormSelectSuccessDistanceStep", Value = 8 })
    SetConfigOption({ Name = "FreeFormSelectRepeatDelay", Value = 0.0 })
    PlaySound({ Name = "/SFX/Menu Sounds/ContractorMenuClose" })
    CloseScreen( GetAllIds( screen.Components ) )
    UnfreezePlayerUnit()
    screen.KeepOpen = false
    OnScreenClosed({ Flag = screen.Name })
end
