--[[
Mod: RewardMenu
Author: hllf & JLove
Version: 28

Intended as an accessibility mod. Places all interactable rewards, NPCs, and other relevant items in a menu, allowing the player to select one and be teleported to it.
Use the mod importer to import this mod.
--]]

OnControlPressed{ "MoveDown", function(triggerArgs)
    if IsScreenOpen("TraitTrayScreen") then
        local rewardsTable = {}
        local curMap = GetMapName({})
        if string.find(curMap, "RoomPreRun") then
            rewardsTable = ProcessTable(MapState.WeaponKits)
        else
            rewardsTable = ProcessTable(ActivatedObjects)
        end
        if TableLength(rewardsTable) > 0 then
            CloseAdvancedTooltipScreen()
            OpenRewardMenu(rewardsTable)
        else
            return
        end
        end
end}

OnControlPressed{ "AdvancedTooltip", function(triggerArgs)
    local rewardsTable = {}
    local curMap = GetMapName({})
    if curMap:find("DeathArea", 1, true) == 1 or curMap:find("E_", 1, true) == 1 then
        rewardsTable = ProcessTable(ActivatedObjects)
        if TableLength(rewardsTable) > 0 then
            OpenRewardMenu(rewardsTable)
        end
    end
end}

local nameToPreviewName = {
["RoomRewardMetaPoint"] = "Darkness",
["RoomRewardMetaPointRunProgress"] = "Darkness (Pitch-Black)",
["MetaPoints"] = "Darkness",
["Gem"] = "Gemstones",
["GemRunProgress"] = "Gemstones (Brilliant)",
["Gems"] = "Gemstones",
["LockKey"] = "Chthonic Key",
["LockKeyRunProgress"] = "Chthonic Key (Fated)",
["Gift"] = "Nectar",
["GiftRunProgress"] = "Nectar (Vintage)",
["RoomRewardMaxHealth"] = "Centaur Heart",
["StackUpgrade"] = "Pom of Power",
["WeaponUpgrade"] = "Daedalus Hammer",
["RoomRewardMoney"] = "Obols",
["Money"] = "Obols",
["SuperLockKey"] = "Titan Blood",
["SuperGem"] = "Diamond",
["SuperGift"] = "Ambrosia",
["HermesUpgrade"] = "Hermes",
["AthenaUpgrade"] = "Athena",
["CerberusKey"] = "Satyr Sack",
["HealthFountain"] = "Fountain",
["HealthFountainAsphodel"] = "Fountain",
["HealthFountainElysium"] = "Fountain",
["HealthFountainStyx"] = "Fountain",
["SwordWeapon"] = "Stygian Blade",
["BowWeapon"] = "Heart-Seeking Bow",
["SpearWeapon"] = "Eternal Spear",
["GunWeapon"] = "Adamant Rail",
["FistWeapon"] = "Twin Fists",
["ShieldWeapon"] = "Shield of Chaos",
["NPC_Cerberus_Field_01"] = "Cerberus",
}

function ProcessTable(objects)
    local table = InitializeObjectList(objects)
    if CurrentRun and CurrentRun.CurrentRoom and not CurrentRun.CurrentRoom.ExitsUnlocked and not (TableLength(ActivatedObjects) > 0) then
        table = AddCure(table)
    end
    table = AddFood(table)
    table = AddObols(table)
    table = AddDarkness(table)
    table = AddGemstones(table)
    table = AddNectar(table)
    table = AddDiamonds(table)
    table = AddFishingPoint(table)
    table = AddGiftRack(table)
    if CurrentRun and CurrentRun.CurrentRoom and CurrentRun.CurrentRoom.ExitsUnlocked then
        table = AddTrove(table)
        table = AddWell(table)
        table = AddPool(table)
    end
    table = AddSkelly(table)
    table = AddEscapeDoor(table)
    if GameState and GameState.Flags and not GameState.Flags.InFlashback then
        table = AddNPCs(table)
    end
    table = AddHouseContractor(table)
    table = AddWretchedBroker(table)
    table = AddHeadChef(table)
    table = AddSackOfObols(table)
    return table
end

function InitializeObjectList(objects)
    local initTable = CollapseTableOrderedByKeys(objects) or {}
    local copy = {}
    for i, v in ipairs(initTable) do
        table.insert(copy, {["ObjectId"] = v.ObjectId, ["Name"] = v.Name})
    end
    return copy
end

function AddCure(objects)
    local NV = GetIdsByType({Name = "PoisonCureFountainStyx" })
    if TableLength(NV) == 0 then
        return objects
    end
    local copy = ShallowCopyTable(objects)
    for ID = #NV, 1, -1 do
        if IsUseable({ Id = NV[ID] }) then
            local cure = {
                ["ObjectId"] = NV[ID],
                ["Name"] = "Mandragora Curing Pool",
            }
            if not ObjectAlreadyPresent(cure, copy) then
                copy = TableInsertAtBeginning(copy, cure)
            end
        end
    end
    return  copy
end

function AddFood(objects)
    local NV = CombineTablesIPairs(GetIdsByType({Name = "HealDropMinor" }), GetIdsByType({Name = "RoomRewardHealDrop" }))
    if TableLength(NV) == 0 then
        return objects
    end
    local copy = ShallowCopyTable(objects)
    NV = GetIdsByType({Name = "HealDropMinor" })
    if TableLength(NV) > 0 then
        for ID = 1, #NV do
            if IsUseable({ Id = NV[ID] }) then
                local food = {
                    ["ObjectId"] = NV[ID],
                    ["Name"] = "Food (Dropped)",
                }
                if not ObjectAlreadyPresent(food, copy) then
                    table.insert(copy, food)
                end
            end
        end
    end
    NV = GetIdsByType({Name = "RoomRewardHealDrop" })
    if TableLength(NV) > 0 then
        for ID = 1, #NV do
            if IsUseable({ Id = NV[ID] }) then
                local food = {
                    ["ObjectId"] = NV[ID],
                    ["Name"] = "Food",
                }
                if not ObjectAlreadyPresent(food, copy) then
                    table.insert(copy, food)
                end
            end
        end
    end
    return  copy
    end

function AddObols(objects)
    local NV = CombineTablesIPairs(GetIdsByType({Name = "RoomRewardMoneyDrop" }), GetIdsByType({Name = "MinorMoneyDrop" }))
    if TableLength(NV) == 0 then
        return objects
    end
    local copy = ShallowCopyTable(objects)
    for ID = 1, #NV do
        if IsUseable({ Id = NV[ID] }) then
            local obols = {
                ["ObjectId"] = NV[ID],
                ["Name"] = "Obols",
            }
            if not ObjectAlreadyPresent(obols, copy) then
                table.insert(copy, obols)
            end
        end
    end
    return  copy
    end

function AddDarkness(objects)
    local NV = CombineTablesIPairs(GetIdsByType({Name = "RoomRewardMetaPointDrop" }), GetIdsByType({Name = "RoomRewardMetaPointDropRunProgress" }))
    if TableLength(NV) == 0 then
        return objects
    end
    local copy = ShallowCopyTable(objects)
    for ID = 1, #NV do
        if IsUseable({ Id = NV[ID] }) then
            local darkness = {
                ["ObjectId"] = NV[ID],
                ["Name"] = "Darkness",
            }
            if not ObjectAlreadyPresent(darkness, copy) then
                table.insert(copy, darkness)
            end
        end
    end
    return  copy
    end

function AddGemstones(objects)
    local NV = CombineTablesIPairs(GetIdsByType({Name = "GemDrop" }), GetIdsByType({Name = "GemDropRunProgress" }))
    if TableLength(NV) == 0 then
        return objects
    end
    local copy = ShallowCopyTable(objects)
    for ID = 1, #NV do
        if IsUseable({ Id = NV[ID] }) then
            local gem = {
                ["ObjectId"] = NV[ID],
                ["Name"] = "Gemstones",
            }
            if not ObjectAlreadyPresent(gem, copy) then
                table.insert(copy, gem)
            end
        end
    end
    return  copy
    end

function AddNectar(objects)
    local NV = CombineTablesIPairs(GetIdsByType({Name = "GiftDrop" }), GetIdsByType({Name = "GiftDropRunProgress" }))
    if TableLength(NV) == 0 then
        return objects
    end
    local copy = ShallowCopyTable(objects)
    for ID = 1, #NV do
        if IsUseable({ Id = NV[ID] }) then
            local nectar = {
                ["ObjectId"] = NV[ID],
                ["Name"] = "Nectar",
            }
            if not ObjectAlreadyPresent(nectar, copy) then
                table.insert(copy, nectar)
            end
        end
    end
    return  copy
    end

function AddDiamonds(objects)
    local NV = GetIdsByType({Name = "SuperGemDrop" })
    if TableLength(NV) == 0 then
        return objects
    end
    local copy = ShallowCopyTable(objects)
    for ID = 1, #NV do
        if IsUseable({ Id = NV[ID] }) then
            local diamond = {
                ["ObjectId"] = NV[ID],
                ["Name"] = "Diamond",
            }
            if not ObjectAlreadyPresent(diamond, copy) then
                table.insert(copy, diamond)
            end
        end
    end
    return  copy
    end

function AddFishingPoint(objects)
if not (CurrentRun.CurrentRoom.ForceFishing and CurrentRun.CurrentRoom.FishingPointId and IsUseable({ Id = CurrentRun.CurrentRoom.FishingPointId })) then
return objects
end
local canFishInEncounter = true
if CurrentRun.CurrentRoom.Encounter and CurrentRun.CurrentRoom.Encounter.BlockFishingBeforeStart and not CurrentRun.CurrentRoom.Encounter.Completed then
canFishInEncounter = false
end
if (CurrentRun and IsCombatEncounterActive( CurrentRun )) or not canFishInEncounter then
return objects
end
local NV = CurrentRun.CurrentRoom.FishingPointId
local copy = ShallowCopyTable(objects)
local fish = {
["ObjectId"] = CurrentRun.CurrentRoom.FishingPointId,
["Name"] = "Fishing Point",
}
if not ObjectAlreadyPresent(fish, copy) then
table.insert(copy, fish)
end
return  copy
end

function AddGiftRack(objects)
local NV = GetIdsByType({Name = "GiftRack" })
if TableLength(NV) == 0 then
return objects
end
local ID = NV[1]
if not IsUseable({ Id = NV[1] }) then
return objects
end
local copy = ShallowCopyTable(objects)
local rack = {
["ObjectId"] = NV[1],
["Name"] = "Keepsake Display Case",
}
if not ObjectAlreadyPresent(rack, copy) then
table.insert(copy, rack)
end
return  copy
	end

function AddTrove(objects)
if not (CurrentRun.CurrentRoom.ChallengeSwitch and IsUseable({ Id = CurrentRun.CurrentRoom.ChallengeSwitch.ObjectId })) then
return objects
end
local NV = CurrentRun.CurrentRoom.ChallengeSwitch.ObjectId
local copy = ShallowCopyTable(objects)
	local switch = {
	["ObjectId"] = CurrentRun.CurrentRoom.ChallengeSwitch.ObjectId,
	["Name"] = "Infernal Trove (" .. (nameToPreviewName[CurrentRun.CurrentRoom.ChallengeSwitch.RewardType] or CurrentRun.CurrentRoom.ChallengeSwitch.RewardType) .. ")",
}
if not ObjectAlreadyPresent(switch, copy) then
table.insert(copy, switch)
end
return  copy
	end
	
function AddWell(objects)
if not (CurrentRun.CurrentRoom.WellShop and IsUseable({ Id = CurrentRun.CurrentRoom.WellShop.ObjectId })) then
return objects
end
local NV = CurrentRun.CurrentRoom.WellShop.ObjectId
local copy = ShallowCopyTable(objects)
local well = {
["ObjectId"] = CurrentRun.CurrentRoom.WellShop.ObjectId,
["Name"] = "Well of Charon",
}
if not ObjectAlreadyPresent(well, copy) then
table.insert(copy, well)
end
return  copy
	end

function AddPool(objects)
if not (CurrentRun.CurrentRoom.SellTraitShop and IsUseable({ Id = CurrentRun.CurrentRoom.SellTraitShop.ObjectId })) then
return objects
end
local NV = CurrentRun.CurrentRoom.SellTraitShop.ObjectId
local copy = ShallowCopyTable(objects)
local pool = {
["ObjectId"] = CurrentRun.CurrentRoom.SellTraitShop.ObjectId,
["Name"] = "Pool of Purging",
}
if not ObjectAlreadyPresent(pool, copy) then
table.insert(copy, pool)
end
return  copy
	end

function AddSkelly(objects)
if not string.find(GetMapName({}), "RoomPreRun") then
return objects
end
local NV = GetIdsByType({Name = "TrainingMelee" })
if TableLength(NV) == 0 then
return objects
end
local ID = NV[1]
if not (ActiveEnemies[NV[1]] and not ActiveEnemies[NV[1]].IsDead) then
return objects
end
local copy = ShallowCopyTable(objects)
local skelly = {
["ObjectId"] = NV[1],
["Name"] = "Skelly",
}
if not ObjectAlreadyPresent(skelly, copy) then
copy = TableInsertAtBeginning(copy, skelly)
end
return  copy
	end

function AddEscapeDoor(objects)
if not string.find(GetMapName({}), "RoomPreRun") then
return objects
end
local NV = GetIdsByType({Name = "NewRunDoor" })
if TableLength(NV) == 0 then
return objects
end
local ID = NV[1]
if not IsUseable({ Id = NV[1] }) then
return objects
end
local copy = ShallowCopyTable(objects)
local window = {
["ObjectId"] = NV[1],
["Name"] = "Escape Window",
}
if not ObjectAlreadyPresent(window, copy) then
table.insert(copy, window)
end
return  copy
	end

function AddNPCs(objects)
    if CurrentRun and IsCombatEncounterActive( CurrentRun ) then
        return objects
    end
    local npcs = CollapseTableOrderedByKeys(ActiveEnemies)
    if TableLength(npcs) == 0 then
        return objects
    end
    local copy = ShallowCopyTable(objects)
    for i = 1, #npcs do
        local skip = false
        if IsUseable({ Id = npcs[i].ObjectId }) then
            local npc = {
                ["ObjectId"] = npcs[i].ObjectId,
                ["Name"] = nameToPreviewName[npcs[i].Name] or npcs[i].Name,
            }
            if npcs[i].Name == "NPC_Hades_01" and GetMapName({}) == "DeathArea" then --Hades in house
                if ActiveEnemies[555686] then --Hades is in garden
                    npc["ObjectId"] = 555686
                elseif GetDistance({ Id = npc["ObjectId"], DestinationId = 422028 }) < 100 then --Hades on his throne
                    npc["DestinationOffsetY"] = 150
                end
            elseif npcs[i].Name == "NPC_Cerberus_01" and GetMapName({}) == "DeathArea" and GetDistance({ Id = npc["ObjectId"], DestinationId = 422028 }) > 500 then --Cerberus not present in house
                skip = true
            elseif npcs[i].Name == "NPC_Cerberus_Field_01" and TableLength(OfferedExitDoors) == 1 and CollapseTable(OfferedExitDoors)[1].Room.Name:find("D_Boss", 1, true) == 1 and CollapseTable(OfferedExitDoors)[1].Room.ChosenRewardType ~= "RoomRewardHealDrop" then --Cerberus in Styx after having been given satyr sack
                skip = true
            end
            if not ObjectAlreadyPresent(npc, copy) and not skip then
                table.insert(copy, npc)
            end
        end
    end
    return  copy
    end

function AddHouseContractor(objects)
if GetMapName({}) ~= "DeathArea" or (GameState and GameState.Flags and GameState.Flags.InFlashback) then
return objects
end
local NV = {210158}
if TableLength(NV) == 0 then
return objects
end
local ID = NV[1]
if not IsUseable({ Id = NV[1] }) then
return objects
end
local copy = ShallowCopyTable(objects)
local contractor = {
["ObjectId"] = NV[1],
["Name"] = "House Contractor",
["DestinationOffsetY"] = 25,
}
if not ObjectAlreadyPresent(contractor, copy) then
table.insert(copy, contractor)
end
return  copy
	end

function AddWretchedBroker(objects)
if GetMapName({}) ~= "DeathArea" or (GameState and GameState.Flags and GameState.Flags.InFlashback) then
return objects
end
local NV = {423390}
if TableLength(NV) == 0 then
return objects
end
local ID = NV[1]
if not IsUseable({ Id = NV[1] }) then
return objects
end
local copy = ShallowCopyTable(objects)
local broker = {
["ObjectId"] = NV[1],
["Name"] = "Wretched Broker",
["DestinationOffsetX"] = -225,
["DestinationOffsetY"] = -100
}
if not ObjectAlreadyPresent(broker, copy) then
table.insert(copy, broker)
end
return  copy
	end

function AddHeadChef(objects)
if GetMapName({}) ~= "DeathArea" or (GameState and GameState.Flags and GameState.Flags.InFlashback) then
return objects
end
local NV = {423399}
if TableLength(NV) == 0 then
return objects
end
local ID = NV[1]
if not IsUseable({ Id = NV[1] }) then
return objects
end
local copy = ShallowCopyTable(objects)
local chef = {
["ObjectId"] = NV[1],
["Name"] = "Head Chef",
}
if not ObjectAlreadyPresent(chef, copy) then
table.insert(copy, chef)
end
return  copy
	end

function AddSackOfObols(objects)
local curMap = GetMapName({})
if not string.find(curMap, "Shop") and not string.find(curMap, "PreBoss") and not string.find(curMap, "D_Hub") then
return objects
end
if not (CurrentRun and CurrentRun.CurrentRoom and CurrentRun.CurrentRoom.Store and CurrentRun.CurrentRoom.Store.SpawnedStoreItems) then
return objects
end
local NV = {}
for k, v in pairs(CurrentRun.CurrentRoom.Store.SpawnedStoreItems) do
if v.Name == "ForbiddenShopItem" then
table.insert(NV, v.ObjectId)
end
end
if TableLength(NV) == 0 then
return objects
end
local ID = NV[1]
if not IsUseable({ Id = NV[1] }) then
return objects
end
local copy = ShallowCopyTable(objects)
local sack = {
["ObjectId"] = NV[1],
["Name"] = "Sack of Obols (Elite)",
}
if not ObjectAlreadyPresent(sack, copy) then
table.insert(copy, sack)
end
return  copy
	end

function ObjectAlreadyPresent(object, objects)
    found = false
    for k, v in ipairs(objects) do
        if object.ObjectId == v.ObjectId then
            found = true
        end
    end
    if CurrentRun and CurrentRun.CurrentRoom and CurrentRun.CurrentRoom.Store and NumUseableObjects(CurrentRun.CurrentRoom.Store.SpawnedStoreItems) > 0 then
        for k, v in pairs(CurrentRun.CurrentRoom.Store.SpawnedStoreItems) do
            if object.ObjectId == v.ObjectId and v.Name ~= "ForbiddenShopItem" then
                found = true
            end
        end
    end
    return found
end

function TableInsertAtBeginning(baseTable, insertValue)
    if baseTable == nil or insertValue == nil then
        return
    end
    local returnTable = {}
    table.insert(returnTable, insertValue)
    for k, v in ipairs(baseTable) do
        table.insert(returnTable, v)
    end
    return returnTable
end

function GetWeaponDisplayConditions(name)
    found = false
    for k, weaponName in ipairs( WeaponSets.HeroMeleeWeapons ) do
        if name == weaponName then
            found = true
        end
    end
    if not found then
        return ""
    end
    if CurrentRun.Hero.Weapons[name] ~= nil then
        if IsWeaponUnused(name) then
            return " (Equipped, Dark Thirst)"
        else
            return " (Equipped)"
        end
    else
        if IsWeaponUnused(name) then
            return " (Dark Thirst)"
        else
            return ""
        end
    end
end

function OpenRewardMenu(rewards)
local screen = { Components = {} }
screen.Name = "BlindAccessibilityRewardMenu"

if IsScreenOpen( screen.Name ) then
return
end
OnScreenOpened({ Flag = screen.Name, PersistCombatUI = false })
HideCombatUI()
FreezePlayerUnit()
EnableShopGamepadCursor()
SetConfigOption({ Name = "FreeFormSelectWrapY", Value = false })
SetConfigOption({ Name = "FreeFormSelectStepDistance", Value = 8 })
SetConfigOption({ Name = "FreeFormSelectSuccessDistanceStep", Value = 8 })
SetConfigOption({ Name = "FreeFormSelectRepeatDelay", Value = 0.6 })
SetConfigOption({ Name = "FreeFormSelectRepeatInterval", Value = 0.1 })
SetConfigOption({ Name = "FreeFormSelecSearchFromId", Value = 0 })

PlaySound({ Name = "/SFX/Menu Sounds/ContractorMenuOpen" })
local components = screen.Components

components.ShopBackgroundDim = CreateScreenComponent({ Name = "rectangle01", Group = "Menu_UI" })
components.CloseButton = CreateScreenComponent({ Name = "ButtonClose", Group = "Menu_UI_Backing", Scale = 0.7 })
Attach({ Id = components.CloseButton.Id, DestinationId = components.ShopBackgroundDim.Id, OffsetX = 0, OffsetY = 440 })
components.CloseButton.OnPressedFunctionName = "CloseRewardMenu"
components.CloseButton.ControlHotkey = "Cancel"

SetScale({ Id = components.ShopBackgroundDim.Id, Fraction = 4 })
SetColor({ Id = components.ShopBackgroundDim.Id, Color = {0, 0, 0, 1} })

CreateRewardButtons(screen, rewards)
screen.KeepOpen = true
thread( HandleWASDInput, screen )
HandleScreenInput( screen )

end

function CreateRewardButtons(screen, rewards)
    local xPos = 960
    local startY = 235
    local yIncrement = 55
    local curY = startY
    local components = screen.Components
    local isFirstButton = true
    if not string.find(GetMapName({}), "RoomPreRun") and GetMapName({}):find("DeathArea", 1, true) ~= 1 and GetMapName({}):find("E_", 1, true) ~= 1 then
        components.statsTextBacking = CreateScreenComponent({ 
            Name = "BlankObstacle", 
            Group = "Menu_UI_Rewards",
            Scale = 1, 
            X = xPos,
            Y = curY
        })
        CreateTextBox({
            Id = components.statsTextBacking.Id,
            Text = "Health: " .. (CurrentRun.Hero.Health or 0) .. "/" .. (CurrentRun.Hero.MaxHealth or 0),
            FontSize = 24,
            Width = 360,
            OffsetX = 0,
            OffsetY = 0,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group = "Menu_UI_Rewards",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        curY = curY + yIncrement
        CreateTextBox({
            Id = components.statsTextBacking.Id,
            Text = "Obols: " .. ((CurrentRun or {Money = 0}).Money or 0),
            FontSize = 24,
            Width = 360,
            OffsetX = 0,
            OffsetY = yIncrement,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group = "Menu_UI_Rewards",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        curY = curY + yIncrement
    end
    for k, reward in pairs(rewards) do
        local displayText = reward.Name
        local buttonKey = "RewardMenuButton" .. k .. displayText
        components[buttonKey] = 
        CreateScreenComponent({ 
            Name = "MarketSlot", 
            Group = "Menu_UI_Rewards",
            Scale = 0.8, 
            X = xPos,
            Y = curY
        })
        components[buttonKey].index = k
        components[buttonKey].reward = reward
        components[buttonKey].OnPressedFunctionName = "GoToReward"
        if reward.Args ~= nil and reward.Args.ForceLootName then
            displayText = reward.Args.ForceLootName:gsub("Upgrade", ""):gsub("Drop", "")
        end
        displayText = nameToPreviewName[displayText:gsub("Drop", ""):gsub("StoreReward", "")] or displayText
        displayText = (displayText .. GetWeaponDisplayConditions(reward.Name)) or displayText
        CreateTextBox({
            Id =components[buttonKey].Id,
            Text =displayText,
            FontSize = 24,
            Width = 720,
            OffsetX = -320,
            OffsetY = 0,
            Color = Color.White,
            Font = "AlegreyaSansSCLight",
            Group="Menu_UI_Rewards",
            ShadowBlur = 0, ShadowColor = {0,0,0,1}, ShadowOffset={0, 2},
            Justification = "Left",
        })
        if isFirstButton then
            TeleportCursor({ OffsetX = xPos, OffsetY = curY })
            isFirstButton = false
        end
        curY = curY + yIncrement
    end
end

function GoToReward(screen, button)
PlaySound({ Name = "/SFX/Menu Sounds/ContractorItemPurchase" })
CloseRewardMenu(screen, button)
local RewardID = nil
RewardID = button.reward.ObjectId
destinationOffsetX = button.reward.DestinationOffsetX or 0
destinationOffsetY = button.reward.DestinationOffsetY or 0
if RewardID  ~= nil then
Teleport({ Id = CurrentRun.Hero.ObjectId, DestinationId = RewardID, OffsetX = destinationOffsetX, OffsetY = destinationOffsetY})
end
end

function CloseRewardMenu( screen, button )
SetConfigOption({ Name = "FreeFormSelectStepDistance", Value = 16 })
SetConfigOption({ Name = "FreeFormSelectSuccessDistanceStep", Value = 8 })
SetConfigOption({ Name = "FreeFormSelectRepeatDelay", Value = 0.0 })

PlaySound({ Name = "/SFX/Menu Sounds/ContractorMenuClose" })
CloseScreen( GetAllIds( screen.Components ) )
ShowCombatUI()
UnfreezePlayerUnit()
screen.KeepOpen = false
OnScreenClosed({ Flag = screen.Name })
end

ModUtil.WrapBaseFunction("ExitNPCPresentation", function(baseFunc, source, args)

	AddInputBlock({ Name = "NPCExit" })
	wait( args.InitialWaitTime or 0 )

	FadeOut({ Color = Color.Black, Duration = args.FadeOutTime or 0.5 })

	PlaySound({ Name = args.InitialExitSound or "/EmptyCue", Delay = 0.7 })

	wait( (args.FadeOutTime or 0.5) + 0.3 )

	if args.DeleteId ~= nil then
		Destroy({ Id = args.DeleteId })
	end

	PlaySound({ Name = args.FootstepSound or "/Leftovers/SFX/FootstepsConcreteMedium" })
	PlaySound({ Name = args.FootstepSound or "/Leftovers/SFX/FootstepsConcreteMedium", Delay = 0.3 })
	PlaySound({ Name = args.MoveSound or "/SFX/Enemy Sounds/Megaera/MegaeraWingFlap", Delay = 0.4 })
	PlaySound({ Name = args.FootstepSound or "/Leftovers/SFX/FootstepsConcreteMedium", Delay = 0.6 })
	PlaySound({ Name = args.FootstepSound or "/Leftovers/SFX/FootstepsConcreteMedium", Delay = 0.9 })
	PlaySound({ Name = args.MoveSound or "/SFX/Enemy Sounds/Megaera/MegaeraWingFlap", Delay = 1.0 })
	if args.UseAdditionalFootstepSounds then
		PlaySound({ Name = args.FootstepSound or "/Leftovers/SFX/FootstepsConcreteMedium", Delay = 1.2 })
		PlaySound({ Name = args.FootstepSound or "/Leftovers/SFX/FootstepsConcreteMedium", Delay = 1.5 })
	end

	if args.UseThanatosExitSound then
		thread( PlayVoiceLines, GlobalVoiceLines.ThanatosSpecialExitVoiceLines, true )
		PlaySound({ Name = "/Leftovers/SFX/BeaconTeleportSFX2", Delay = 2.2 })
	end

	LockCamera({ Id = CurrentRun.Hero.ObjectId })
	Teleport({ Id = args.ObjectId or source.ObjectId, DestinationId = args.TeleportToId })
	-- Function modification by hllf: Insert next line
	UseableOff({ Id = args.ObjectId or source.ObjectId })
	if args.AltObjectId ~= nil then
		Teleport({ Id = args.AltObjectId, DestinationId = args.TeleportToId })
		-- Function modification by hllf: Insert next line
		UseableOff({ Id = args.AltObjectId })
	end

	wait( args.FullFadeTime or 1.5 )

	thread( PlayVoiceLines, HeroVoiceLines[args.HeroVoiceLines] )

	FadeIn({ Duration = args.FadeInTime or 1.0 })

	RemoveInputBlock({ Name = "NPCExit" })

	PlaySound({ Name = args.EndSound or "/EmptyCue", Delay = 0.3 })

	if args.EndUnlockText ~= nil then
		thread( DisplayUnlockText, {
			TitleText = args.EndUnlockText.."_Title",
			SubtitleText = args.EndUnlockText.."_Subtitle",
			AnimationName = args.AnimationName or "LocationTextBGGeneric",
			AnimationOutName = args.AnimationOutName or "LocationTextBGGenericOut",
			FontScale = args.FontScale or 1.0,
			SubtitleData = { LuaKey = "TempTextData", LuaValue = { Name = CurrentRun.CurrentRoom.EndUnlockText }},
			Delay = 0.6,
		})
	elseif args.EndUnlockTextTable ~= nil then
		local text = GetRandomValue( args.EndUnlockTextTable )
		thread( DisplayUnlockText, {
			TitleText = text.."_Title",
			SubtitleText = text.."_Subtitle",
			AnimationName = args.AnimationName or "LocationTextBGGeneric",
			AnimationOutName = args.AnimationOutName or "LocationTextBGGenericOut",
			FontScale = args.FontScale or 1.0,
			SubtitleData = { LuaKey = "TempTextData", LuaValue = { Name = CurrentRun.CurrentRoom.EndUnlockText }},
			Delay = 0.6,
		})
	end

end)
